
% The Boston housing dataset, from the UCI Machine Learning repository

raw_data    = load('housing.dat');
sel_rows    = find(raw_data(:,9) < 24);
raw_data    = raw_data(sel_rows, :);
log_vars    = [3 8 9 10];
raw_data(:,log_vars) = log(raw_data(:, log_vars));
sample_size = size(raw_data, 1);

num_latents  = 4;
num_observed = 11;

n_types = ones(num_latents + num_observed, 1);
n_types(1:num_latents) = 0;

graph = zeros(num_latents + num_observed);

% Accessibility
for i = 1:2
    graph(num_latents + i, 1) = 1;
end
% Neighborhood I
for i = 3:5
    graph(num_latents + i, 2) = 1;
end
% Neighborhood II
for i = 6:9
    graph(num_latents + i, 3) = 1;
end
% Structural
for i = 10:11
    graph(num_latents + i, 4) = 1;
end

%graph(2, 1) = 1;
%graph(3, 1) = 1; graph(3, 2) = 1;
%graph(4, 1) = 1; graph(4, 2) = 1; graph(4, 3) = 1;

% Order: 1 Access, 4 Structural, 3 N2, 2 N1
order_latent = [1 4 3 2];
for l = 2:4
    for l0 = 1:(l - 1)
        graph(order_latent(l), order_latent(l0)) = 1;
    end
end

raw_data = raw_data(:,[8 9 1 2 12 3 10 11 13 6 7]);

% Accessibility:
%
% 8. DIS: weighted distances to five Boston employment centres (LOG)
% 9. RAD: index of accessibility to radial highways            (LOG)
%
% Neighborhood I (highly skewed):
%
% 1.  CRIM: per capita crime rate by town
% 2.  ZN: proportion of residential land zoned for lots over 25,000 sq.ft.
% 12. B: 1000(Bk - 0.63)^2 where Bk is the proportion of blacks by town
%
% Neighborhood II (moderately skewed):
%
% 3.  INDUS: proportion of non-retail business acres per town (LOG)
% 10. TAX: full-value property-tax rate per $10,000 (LOG)
% 11. PTRATIO: pupil-teacher ratio by town
% 13. LSTAT: % lower status of the population
%
% Structural:
%
% 6. RM: average number of rooms per dwelling
% 7. AGE: proportion of owner-occupied units built prior to 1940 
%
% Air pollution: (REMOVED)
%
% 5. NOX: nitric oxides concentration (parts per 10 million)
% ?. PART: particulate concentrations (not in the data??)
%
% Target:
%
% 14. MEDV: Median value of owner-occupied homes in $1000's

% Verify if cross-validation files exist
NUM_FOLDS = 5;
cv_file = fopen('housing_a.cv', 'r');
if cv_file == -1 % Create file if it doesn't exist
   cv_file = fopen('housing_a.cv', 'w');
   cv_matrix = get_cv_matrix(sample_size, NUM_FOLDS);
   for f = 1:NUM_FOLDS
       lcv = length(cv_matrix{f});
       for n = 1:lcv
           fprintf(cv_file, '%d ', cv_matrix{f}(n));             
       end
       fprintf(cv_file, '\n');
   end
   fclose(cv_file);
   cv_file = fopen('housing_a.cv', 'r');
end

% Read test points from cross-validation file
all_test_points = cell(NUM_FOLDS, 1);
cv_matrix = get_cv_matrix(sample_size, NUM_FOLDS); % Just to get sizes
for f = 1:NUM_FOLDS
   lcv = length(cv_matrix{f});
   data_line = textscan(cv_file, '%d', lcv);
   all_test_points{f} = data_line{1}';
end
fclose(cv_file);

all_train_points = cell(NUM_FOLDS, 1);
for f = 1:NUM_FOLDS
    all_train_points{f} = setdiff(1:sample_size, all_test_points{f});
end

%test_points = test_points{1}; train_points = train_points{1};

test_vars = [];         % Which variables are going to be predicted
anchors   = [1 3 7 10]; % Variables of single coefficient, which are set to
                        % have unit slop loading and zero intercept.

latent_data = [];
LOG_NAME    = 'mse-gpr-housing.log';
num_x_mix   = 5; 

% Generate missing at random matrix
if exist('housing_a.mars.mat', 'file')
   load('housing_a.mars.mat', 'mars_matrix');
else
   mars_matrix = rand(size(raw_data)) > 0.5;
   mars_matrix(:, test_vars) = 0;
   save 'housing_a.mars.mat' 'mars_matrix'
end

