%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Generate a synthetic dataset with traint/test split from a simple latent
% structure X1 -> X2, each latent variable with three indicators.

num_latents  = 2;
sample_size  = 150;

num_observed = num_latents * 3;

raw_data     = zeros(sample_size, num_observed);
latent_data  = zeros(sample_size, num_latents);
exo_hyper    = ones(num_latents + num_observed, 1);
coeff        = ones(num_observed, 1);

graph = zeros(num_latents + num_observed);
for x = 1:num_latents
    for y = 1:3
        graph(num_latents + (x - 1) * 3 + y, x) = 1;
        coeff((x - 1) * 3 + y) = x;
    end
    for x2 = (x + 1):num_latents
        graph(x2, x) = 1;
    end
end

for d = 1:sample_size
    for x = 1:num_latents
        x_data = randn() * sqrt(exo_hyper(1));
        for x2 = 1:(x - 1)
            x_data = x_data + 4 * latent_data(d, x2)^2;
        end
        latent_data(d, x) = x_data;
    end
    for y = 1:num_observed
        l = 1 + floor((y - 1) / 3);
        raw_data(d, y) = coeff(y) * latent_data(d, l) + randn() * sqrt(exo_hyper(num_latents + y));
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Generates information used by test_gpsem_*

n_types = ones(num_latents + num_observed, 1);
n_types(1:num_latents) = 0;

test_points  = []; % Change if cross-validation desired
test_vars    = []; % Which variables are going to be predicted
train_points = setdiff(1:sample_size, test_points);

anchors = [1 4]; % Variables of single coefficient, which are set to
                 % have unit slop loading and zero intercept.

NUM_FOLDS = 1;

all_train_points = cell(5, 1);
for i = 1:5
    all_train_points{i} = 1:sample_size;
end
all_test_points = cell(5, 1);
for i = 1:5
    all_test_points{i} = [];
end

LOG_NAME = 'synth.log';
num_x_mix = 2;
