% Simple script that assumes some dataset is loaded with an indication of 
% training and test points. It then prepares logs for cross-validation.

% First, run a script that will generated the required data structures. We
% provide a few demos: load_synth, load_depress, load_consumer, load_house
% and load_abalone.

% In all of these demos, 5-fold cross validation is setup. Change
% NUM_FOLDS to a smaller number if you want the experiment to finish
% earlier.

% This model uses a quadratic polynomial regression model instead of 
% general (infinite-dimensional) Gaussian processes.

num_mcmc_samples = 20000;
burn_in          = 2000;

all_mse  = zeros(NUM_FOLDS, length(test_vars));
mse_file = fopen(strcat(LOG_NAME, '.finite'), 'w');

% Comment out next line in order to use the data with missing entries
mars_matrix = zeros(size(raw_data));
% t-distributed measurement error?
use_t = false;

for f = 1:NUM_FOLDS
    
    train_points = all_train_points{f}; %#ok<USENS>
    test_points  = all_test_points{f};  %#ok<USENS>
    data = normalize_training(raw_data, train_points, mars_matrix);
    if isempty(test_vars)
       input_data = data(train_points, :);
       mars_matrix = zeros(size(input_data));
       test_values = [];
    else
       input_data = data; if sum(sum(mars_matrix))> 0, input_data(mars_matrix) = 0; end
       test_values = zeros(length(test_points) * length(test_vars), 2);
       tp = 1;
       for t = test_points    
           for v = test_vars    
               test_values(tp, 1) = t;
               test_values(tp, 2) = v;
               tp = tp + 1;
           end
       end
    end        

    [test_pred, sampled_x, sampled_f, weights, sampled_lin, ...
          sampled_xerr_var, sampled_yerr_var] = ...
              run_finite_mcmc(num_mcmc_samples, graph, input_data, n_types, ...
                              num_x_mix, ...
                              anchors, test_values, use_t, latent_data);

    copyfile('sample_x.txt', strcat('sample_x.', int2str(f), '.log'));
    copyfile('sample_y.txt', strcat('sample_y.', int2str(f), '.log'));
    copyfile('sample_lambda.txt', strcat('sample_lambda.', int2str(f), '.log'));
    copyfile('sample_yvar.txt', strcat('sample_yvar.', int2str(f), '.log'));
    copyfile('sample_yups.txt', strcat('sample_yups.', int2str(f), '.log'));
    copyfile('sample_z.txt', strcat('sample_z.', int2str(f), '.log'));
    copyfile('sample_xvar.txt', strcat('sample_xvar.', int2str(f), '.log'));
    copyfile('sample_exo.txt', strcat('sample_exo.', int2str(f), '.log'));
    copyfile('sample_w.txt', strcat('sample_w.', int2str(f), '.log'));
    
    clear sampled_x; clear sampled_f;
end

fclose(mse_file);
