% kernel_logprior:
%
% Evaluate the kernel logprior for covariance function 'covfunc' of 
% hyperparameter #p at value 'value'.

function logdensity = kernel_logprior(covfunc, p, value)

kernel_testvalid(covfunc);

% First parameter: prior for global component variance is a mixture of
% two gammas as suggested in Gramacy and Lee (JASA, 2008): 0.5 probability
% of a Gamma(1, 20) and 0.5 probability of a Gamma(10, 10)

%max_v = 50;

if p == 1
   logdensity = log(0.5 * gammapdf(value, 1, 20) + 0.5 * gammapdf(value, 10, 10));
   %m = 3; v = 9;
   %logdensity = -0.5 * (log(value) - m)^2 / v - log(value) - 0.5 * log(v) - 0.5 * log(2 * pi);
   return
end

% Second parameter: prior for signal variance: same

if p == 2
   logdensity = log(0.5 * gammapdf(value, 1, 20) + 0.5 * gammapdf(value, 10, 10));
   %m = 3; v = 9;
   %logdensity = -0.5 * (log(value) - m)^2 / v - log(value) - 0.5 * log(v) - 0.5 * log(2 * pi);
   return
end

% Third parameter: a fixed ``nuggett''

if p == 3
   %logdensity = 0;
   logdensity = log(0.5 * gammapdf(value, 1, 20) + 0.5 * gammapdf(value, 10, 10));
   %m = 3; v = 9;
   %logdensity = -0.5 * (log(value) - m)^2 / v - log(value) - 0.5 * log(v) - 0.5 * log(2 * pi);
   return
end

% Raise error

error('COV:PARAM', 'Invalid call of kernel_logprior: parameter does not exist');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function d = gammapdf(value, alpha, beta)

d = value^(alpha - 1) * exp(-value / beta) / (gamma(alpha) * beta^alpha);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end
