% kernel_prop:
%
% Proposal for new kernel hyperparameters.
%
% Input:
%
% - covfunc: string describing the covariance function
% - p: parameter id
% - value: current parameter value.
%
% Output:
%
% - new_value: the proposed new value;
% - logfwd: log of the density function of the proposal value -> new_value
% - logbkw: log of the density function of the proposal new_value -> value


function [new_value logfwd logbkw] = kernel_prop(covfunc, p, value)

kernel_testvalid(covfunc);

if p < 1 || p > kernel_nparam(covfunc)
   error('COV:INVALID', 'Invalid hyperparameter id');
end

%window    = 0.1;

%u         = rand();
%new_value = ((1 - window) + u * 2 * window) * value;
%logfwd    = log(1 / (2 * window * value));
%logbkw    = log(1 / (2 * window * new_value));

%u         = window * (2 * rand() - 1);
%new_value = u + value;
%logfwd    = log(1 / (2 * window));
%logbkw    = log(1 / (2 * window));

ratio     = 7.5/10;
window    = value / ratio - value * ratio;
new_value = value * ratio + rand() * window;
logfwd    = log(1 / window);
window_bk = new_value / ratio - new_value * ratio;
logbkw    = log(1 / window_bk);

