% run_gpsem_mcmc_full:
%
% Simple function to run the MCMC algorithm for the full GP model using
% some default parameters.
%
% Mostly for comparisons only: not a sane thing to do with one has more
% than a few hundred points.

function [test_pred, sampled_x, sampled_f, sampled_lin, ...
          sampled_xerr_var, sampled_yerr_var, sampled_kernel_hyper] = ...
              run_gpsem_mcmc_full(num_mcmc_samples, graph, data, n_types, ...
                                  num_x_mix, ...
                                  anchors, test_values, latent_data, ...
                                  BATCH_SIZE, use_starting, initial_mcmc_samples)

num_x = sum(n_types == 0); num_y = sum(n_types == 1);
first_y = find(n_types == 1, 1); 
other_half = n_types(first_y:length(n_types));
if sum(other_half) ~= length(other_half)
   error('Latent variables should precede all observed variables');
end

sample_size = size(data, 1);
covfunc = {'covSum', {'covSEiso','covNoise'}};
lin_var = 5; % 'Vague' prior
hyper_x_var = [2; 1];
hyper_y_var = hyper_x_var;
fixed_hyper_kernel = [0; 0; 1];
hyper_kernel_values_fix = [1; 1; 1.e-4];
fixed_yerr = zeros(num_y, 1); yerr = ones(num_y, 1);
fixed_xerr = zeros(num_x, 1); xerr = ones(num_x, 1);
fixed_loadings = cell(num_y, 1);
for i = 1:num_y
    num_parents = sum(graph(num_x + i, :));
    fixed_loadings{i} = zeros(num_parents + 1, 1);
end
for y = anchors
    num_parents = sum(graph(num_x + y, :));
    if num_parents ~=1
       error('Anchors should have one latent parent only!');
    end
    if num_parents == 0
       error('Observed variables should have at least one latent parent!');
    end
    fixed_loadings{y} = ones(2, 1);
end

hyper_x_mix = cell(num_x, 3);
for x = 1:num_x
    hyper_x_mix{x, 1} = ones(num_x_mix, 1) * 10 / num_x_mix;
    hyper_x_mix{x, 2} = 10;
    hyper_x_mix{x, 3} = [2; 1];
end

if use_starting
    while true
        % Initializating using the cheaper finite GP model
        [x_start, lambda_mean, var_y, ...
          hyper_kernel_values, var_x, x_marg, z_start] = ...
                initialize_gpsem_finite(initial_mcmc_samples, num_x_mix, graph, data, n_types, ...
                                        anchors, test_values, covfunc);
        if mean(var_y) < 100
           break
        end
    end
else
    x_start = []; lambda_mean = []; var_y = []; 
    hyper_kernel_values = []; var_x = []; x_marg = []; z_start = [];
end


% Now, do the sampling      

[test_pred, sampled_x, sampled_f, sampled_lin, sampled_kernel_hyper, ...
 sampled_xerr_var, sampled_xmix, sampled_x_z, sampled_yerr_var] = ...
          mcmc_gpsem_full(num_mcmc_samples, graph, n_types, data, covfunc, lin_var, ...
                     hyper_x_var, hyper_x_mix, hyper_y_var, ...
                     fixed_hyper_kernel, hyper_kernel_values_fix, ...
                     fixed_yerr, yerr, fixed_xerr, xerr, fixed_loadings, [], ...
                     test_values, ...
                     use_starting, x_start, lambda_mean, var_y, ...
                     hyper_kernel_values, var_x, x_marg, z_start, BATCH_SIZE);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Do simple reality check: in synthetic models, we know the true latent
% values. Those can be passed as the ``latent_data'' vector. If
% ``latent_data'' is available, we plot a comparison of the true X1 x X2
% scatterplot (where X1 and X2 are the first two latent variables) against
% the scatterplot of the posterior expected values of X1 and X2.

                 
if ~isempty(latent_data)                 
   [B,IX] = sort(latent_data(:,1)); % Sort points by the true values of X1
else
   [B,IX] = sort(mean(sampled_x(:, :, 1))); % Sort points by the estimated values of X1
end

% Get average function, sorted by the original ranking of the latent
% variables

if size(sampled_x, 3) == 1 % If just one latent variable, skip
   return
end

pred_latent = zeros(sample_size, 2);
for d = 1:sample_size
    pred_latent(d, 1) = mean(sampled_x(:, d, 1));
    pred_latent(d, 2) = mean(sampled_x(:, d, 2));
end

% Plot true values
if ~isempty(latent_data)
    s1 = subplot(2, 1, 1);
    scatter((latent_data(IX, 1) - mean(latent_data(IX, 1))) / sqrt(var(latent_data(IX, 1))), ...
            (latent_data(IX, 2) - mean(latent_data(IX, 1))) / sqrt(var(latent_data(IX, 2))));
    title(s1, 'X1 x X2, true values');        
else
    s1 = subplot(2, 1, 1); title(s1, 'X1 x X2, true values: UNAVAILABLE');
end
% Plot estimates
s2 = subplot(2, 1, 2);
scatter((pred_latent(IX, 1) - mean(pred_latent(IX, 1))) / sqrt(var(pred_latent(IX, 1))), ...
        (pred_latent(IX, 2) - mean(pred_latent(IX, 2))) / sqrt(var(pred_latent(IX, 2))));
title(s2, 'X1 x X2, posterior expected values');    
