% run_mog_mcmc:
%
% Simple function to test the MCMC algorithm. It samples also error term
% variances and kernel hyperparameters.

function [test_pred, sampled_x, sampled_lin, sampled_yerr_var, ...
          sampled_xmix_pi, sampled_xmix_mu, sampled_xmix_invcov, sampled_x_z] = ...
              run_mog_mcmc(num_mcmc_samples, K, graph, data, n_types, ...
                           anchors, test_values, latent_data)

num_x = sum(n_types == 0); num_y = sum(n_types == 1);
first_y = find(n_types == 1, 1); 
other_half = n_types(first_y:length(n_types));
if sum(other_half) ~= length(other_half)
   error('Latent variables should precede all observed variables');
end

sample_size = size(data, 1);
lin_var = 100;
%hyper_x_var = [1; 10];
%hyper_y_var = hyper_x_var;
hyper_y_var = [2; 2];
fixed_loadings = cell(num_y, 1);
for i = 1:num_y
    num_parents = sum(graph(num_x + i, :));
    fixed_loadings{i} = zeros(num_parents + 1, 1);
end
for y = anchors
    num_parents = sum(graph(num_x + i, :));
    if num_parents ~=1
       error('Anchors should have one latent parent only!');
    end
    if num_parents == 0
       error('Observed variables should have at least one latent parent!');
    end
    fixed_loadings{y} = ones(2, 1);
end

invcov_delta = 10;
hyper_x_mean_invcov = diag(ones(num_x, 1)) * 0.01;
invcov_U = eye(num_x) * invcov_delta;
alpha_w = 10;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INITIALIZE some of the hyperparameters by variational optimization

use_starting = 0;
if use_starting
    k = 5;
    num_y = sum(n_types);
    f_parents = cell(num_y, 1);
    f_intercepts = zeros(num_y, 1);
    y_pos = 1;
    for y = 1:num_y
        while n_types(y_pos) ~= 1
            y_pos = y_pos + 1;
        end
        f_parents{y} = zeros(length(sum(graph(y_pos, :))), 1);
    end
    for x = 1:num_x
        f_parents{3 * (x - 1) + 1, 1} = 1;
        f_intercepts(3 * (x - 1) + 1) = 1;
    end
    [x_means, q_xs, q_s, q_xmean, q_xinvcov, q_pi] = ...
              initialize_gpsem_variational(1, graph, data, n_types, k, ...
                                           f_parents, f_intercepts, covfunc);
    x_marg = cell(num_x, 3);
    z_start = cell(num_x, sample_size);
    for x = 1:num_x
        num_k = length(q_pi);        
        x_marg{x} = zeros(num_k, 3);
        for s = 1:num_k
            x_marg{x}(s, 1) = q_pi(s);
            x_marg{x}(s, 2) = q_xmean{s, 1}(x);
            cov_x = inv(q_xinvcov{s, 2} * q_xinvcov{s, 1});
            x_marg{x}(s, 3) = cov_x(x, x);
        end
        for d = 1:sample_size
           z_start{x, d} = q_s(d, :);
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Now, do the SAMPLING

% Test set indicators
[test_pred, sampled_x, sampled_lin, sampled_yerr_var, ...
 sampled_xmix_pi, sampled_xmix_mu, sampled_xmix_invcov, sampled_x_z] = ...
      mcmc_mog(num_mcmc_samples, K, graph, n_types, data, lin_var, ...
               alpha_w, ...
               hyper_x_mean_invcov, invcov_delta, invcov_U, hyper_y_var, ... 
               fixed_loadings, [], test_values);
